
	BSS
size		dc.w	0
sinetable_ptr	dc.l	0
angle_mask	dc.w	0
	CODE

	xdef	_library
	xdef	triglib@0000
	xdef	triglib@0001
	xdef	triglib@0002
	xdef	triglib@0003
	xdef	triglib@0004

;*****************************************************

;---------------------------------------------------------------------
;
; init(a0,d1,d3)
;
; Function: Generate a sine table of d1*4 words
;
; Input:    A0.L = Pointer to sine table memory area
;           D1.W = Size of 1/4th of table in #angles. *Must* be a
;                   power of two.
; Output:   Nothing
; Destroys: Nothing
; Size:     130 bytes
;
;---------------------------------------------------------------------
R	EQU	32767	;Should be something high, preferrably 2^x-1 (highest: around 32767)
MAG	EQU	16384	;2^14
K	EQU	38603	;6*pi*2^11

triglib@0000:
init:
	movem.l		a0-a1/d0-d4,-(sp)
	move.l		a0,sinetable_ptr
	move.w		d1,size
	lsl.w		#1,d1
	move.w		#0,(a0)		;Start value #1
	move.w		#R,0(a0,d1.w)	;Start value #2
	move.w		#K,d2
	bsr.s		\calc		;Generate default table (radius=32767)

	move.w		d1,d2
	lsl.w		#1,d2		;d2=2*SIZE
	move.w		d2,d4
	lsl.w		#1,d4		;d4=4*SIZE
	move.w		d4,angle_mask
	sub.w		#1,angle_mask

	lea		2(a0,d2.w),a1	;a1=Start adress for right-to-left *** +2 ***
\loop	move.w		(a0)+,d0	;Get 1st value
	move.w		d0,-(a1)	;Store 2nd value
	neg.w		d0
	move.w		d0,-2(a0,d2.w)	;Store 3rd value
	move.w		d0,0(a1,d2.w)	;Store 4th value
	subq.w		#2,d1
	bge.s		\loop

	movem.l		(sp)+,a0-a1/d0-d4
	rts

\calc	cmp.w		#2,d1			;Minimum stepsize=2 (one word)
	beq.s		\done
	movem.l		a0/d1-d2,-(sp)			;Recursive programming ;-)
	lsr.w		#2,d2			;K=K/4
	move.w		#2*MAG,d0
	sub.w		d2,d0			:d0=2*MAG-K
	move.w		(a0),d4			;d4=SinA
	add.w		0(a0,d1.w),d4		;d4=SinA+SinB
	mulu.w		#MAG,d4
	divu.w		d0,d4			;d4=SinC
	lsr.w		#1,d1			;Halve stepsize
	move.w		d4,0(a0,d1.w)		;Store value between the two older values
	bsr.s		\calc			;Calculate new value between SinA and SinC
	add.l		d1,a0			;Next left-to-right adress
	bsr.s		\calc			;Calculate new value between SinC and SinB
	movem.l		(sp)+,a0/d1-d2			;Recursive programming
\done	rts

;---------------------------------------------------------------------
;
; distance(D0,D1)
;
; Function: Calculate the diagonal of a rectangle with the sides D0
;           and D1.
;
; Input:    D0.W = X side length
;           D1.W = Y side length
; Output:   D2.W = sqrrt(X^2+Y^2)
; Destroys: Nothing
; Size:     36 bytes
;
;---------------------------------------------------------------------
triglib@0001:
;distance:
	movem.l		d0-d1,-(sp)

\absx	neg.w		d0
	blt.s		\absx		;Get abs(x)
\absy	neg.w		d1
	blt.s		\absy		;Get abs(y)

	mulu.w		d0,d0		;d0=x^2
	mulu.w		d1,d1		;d1=y^2
	add.l		d0,d1		;d1=x^2+y^2

	moveq		#1,d0		;Odd number counter
	moveq		#0,d2		;Returned value

\loop	addq.l		#2,d0
	addq.l		#1,d2
	sub.l		d0,d1
	bge.s		\loop

	movem.l		(sp)+,d0-d1
	rts

;---------------------------------------------------------------------
;
; arctan(D0,D1,D2)
;
; Function: Calculate arctan(D1/D0)
;
; Input:    D0.W = Delta-X
;           D1.W = Delta-Y
;           D2.W = 0 to return negative angles, otherwise positive
; Output:   D2.W = arctan(D1/D0)
; Destroys: Nothing
; Size:     102 bytes
;
; Note: The error is 1-2 "degrees".
;
;---------------------------------------------------------------------
triglib@0002:
arctan:
	movem.l		a0-a1/d3-d7,-(sp)

	move.l		sinetable_ptr,a0
	move.w		size,d7
	lsl.w		#1,d7
	lea		0(a0,d7.w),a1

	movem.l		d0-d2,-(sp)

\absx	neg.w		d0
	blt.s		\absx		;Get abs(x)
\absy	neg.w		d1
	blt.s		\absy		;Get abs(y)

	moveq		#0,d2

\gloop	move.w		0(a0,d2.w),d4	;X1	(d0=X2)
	move.w		0(a1,d2.w),d5	;Y1	(d1=Y2)

	mulu.w		d0,d4		;d4=X1*Y2
	mulu.w		d1,d5		;d5=X2*Y1

	lsr.w		#1,d7
	cmp.l		d4,d5
	beq.s		\done	;X1*Y2=X2*Y1
	blt.s		\left
\right	add.w		d7,d2	;X1*Y2>X2*Y1 - current angle is to high
	bra.s		\ok
\left	sub.w		d7,d2	;X1*Y2<X2*Y1 - current angle is to low

\ok	btst.b		#1,d2
	beq.s		\gloop		;LSB in d6 is not set - we can take smaller steps!

\done	move.w		size,d7
	lsl.w		#1,d7

	lsr.w		#1,d2

	movem.l		(sp)+,d0-d1/d4


	tst.w		d0
	bge.s		\xpos
	neg.w		d2
	add.w		d7,d2		;X<0 - angle=180-angle

\xpos	tst.w		d1
	bge.s		\ypos
	lsl.w		#1,d7
	neg.w		d2
	tst.w		d4
	beq.s		\ypos		;Y<0, but we want negative angles!
	add.w		d7,d2		;Y<0 - angle=360-angle

\ypos	movem.l		(sp)+,a0-a1/d3-d7
	rts



;---------------------------------------------------------------------
;
; sin()
;
; Function: Calculate sin(D1)
;
; Input:    D0.W = "angle"
;           D1.W = Radius of circle in pixels
; Output:   D2.W = sin(D1)
; Destroys: Nothing
; Size:     36 bytes
;
;---------------------------------------------------------------------
triglib@0003:
sin:
	movem.l		a0/d0-d1,-(sp)

	move.l		sinetable_ptr,a0
	asl.w		#1,d0
	and.w		angle_mask,d0
	move.w		0(a0,d0.w),d2

;	lsl.w		#1,d1
	addq.w		#1,d1
	muls.w		d1,d2
	asl.l		#1,d2
	swap		d2

	movem.l		(sp)+,a0/d0-d1
	rts

;---------------------------------------------------------------------
;
; cos()
;
; Function: Calculate cos(D1)
;
; Input:    D0.W = "angle"
;           D1.W = Radius of circle in pixels
; Output:   D2.W = cos(D1)
; Destroys: Nothing
; Size:     Unknown
;
;---------------------------------------------------------------------
triglib@0004:
cos:
	add.w		size,d0
	bsr		sin
	sub.w		size,d0
	rts

;********************* Variables *******************

;************** End of Fargo program ****************

_library	dc.b	"triglib",0

;*****************************************************

	end
